class GDAXClient
  include Log::Dependency

  setting :url

  configure :client

  def self.build
    instance = new
    Settings.set(instance)
    instance
  end

  def get(endpoint)
    uri = URI(url + endpoint)

    logger.trace "Sending HTTP GET request (URI: #{uri})"

    response = Net::HTTP.get_response(uri)

    logger.debug "HTTP GET response received (URI: #{uri}, Code: #{response.code}, Message: #{response.message})"

    response
  end

  class Substitute
    attr_accessor :http_code
    attr_accessor :http_message
    attr_accessor :response_body

    def self.build
      new
    end

    def get(endpoint)
      self.http_code ||= '200'
      self.http_message ||= 'OK'
      self.response_body ||= {}

      Response.new(http_code, http_message, response_body)
    end

    Response = Struct.new :code, :message, :body
  end
end
