class GDAXClient
  module HTTP
    module ProductTicker
      class Get
        include Log::Dependency

        dependency :client, GDAXClient

        configure :get_ticker

        def self.call(product_id)
          instance = build
          instance.(product_id)
        end

        def self.build
          instance = new
          GDAXClient.configure(instance)
          instance
        end

        def call(product_id)
          endpoint = "/products/#{product_id}/ticker"

          logger.trace "Getting ticker (ProductID: #{product_id}, Endpoint: #{endpoint})"

          raw_response = client.get(endpoint)

          if raw_response.code == '200'
            logger.debug "Retrieved ticker (ProductID: #{product_id}, Endpoint: #{endpoint}, Status: #{raw_response.code}, Message: #{raw_response.message})"
            response = Transform::Read.(raw_response.body, Response, :json)
          else
            logger.debug "Failed to retrieve ticker (ProductID: #{product_id}, Endpoint: #{endpoint}, Status: #{raw_response.code}, Message: #{raw_response.message})"
            response = Response.new
          end

          response.status_code = raw_response.code
          response.message = raw_response.message

          response
        end

        class Substitute
          attr_writer :response

          def self.build
            new
          end

          def call(product_id)
            response
          end

          def response
            @response ||= Response.new
          end
        end
      end
    end
  end
end
